#ifndef _XC3S_Config_H
#define _XC3S_Config_H

#include "Prefix.h"
#include "XC3S_Register.h"
#include "EARTH_PT.h"

namespace EARTH {
namespace XC3S {
	class Register;

	class Config {
	public:
		Config(XC3S::Register *register_) : mRegister(register_)
		{
			_ASSERTE(mRegister);

			mPTn = 0;
		}

		~Config()
		{
		}

		void SetPTn(uint ptn)
		{
			_ASSERT((ptn == 1) || (ptn == 2));
			mPTn = ptn;
		}

		status CheckBus()
		{
			for (uint i=0; i<2; i++) {
				uint data = (i == 0) ? 0x00000000 : 0xffffffff;
				for (uint j=0; j<32; j++) {
					uint writeData = data ^ (BIT_1(1) << i);
					uint readData;

					mRegister->WriteConfig4(A_CONFIG_DATA, writeData);
					mRegister->ReadConfig4 (A_CONFIG_DATA, &readData);

					if (readData != writeData) return PT::STATUS_PCI_BUS_ERROR;
				}
			}

			uint writeData = 0;
			uint readData;

			mRegister->WriteConfig4(A_CONFIG_DATA, writeData);
			mRegister->ReadConfig4 (A_CONFIG_DATA, &readData);

			if (readData != writeData) return PT::STATUS_PCI_BUS_ERROR;

			for (uint i=0; i<4; i++) {
				enum {
					DATA = 0xff
				};

				mRegister->WriteConfig1(A_CONFIG_DATA+i, DATA);
				mRegister->ReadConfig4 (A_CONFIG_DATA, &readData);

				writeData <<= 8;
				writeData  |= DATA;

				if (readData != writeData) return PT::STATUS_PCI_BUS_ERROR;
			}

			return PT::STATUS_OK;
		}

		void GetVersion(uint *version)
		{
			_ASSERTE(version);
			mRegister->ReadConfig4(A_CONFIG_VERSION, version);
		}

		struct Abort {
			bool Target,
				 Master;
		};

		void ResetAbort(const Abort *abort)
		{
			_ASSERTE(abort);
			ushort data =
				  (abort->Target ? 1 : 0) << 12
				| (abort->Master ? 1 : 0) << 13;

			mRegister->WriteConfig2(0x06, data);
		}

		void GetAbort(Abort *abort)
		{
			ushort data;

			mRegister->ReadConfig2(0x06, &data);

			_ASSERTE(abort);
			abort->Target = BIT_SHIFT_MASK(data, 12, 1) ? true : false;
			abort->Master = BIT_SHIFT_MASK(data, 13, 1) ? true : false;
		}

		struct Enable {
			bool Memory,
				 Master;
		};

		status SetEnable(const Enable *enable)
		{
			_ASSERTE(enable);
			ushort data =
				  (enable->Memory ? 1 : 0) << 1
				| (enable->Master ? 1 : 0) << 2;

			status status = mRegister->WriteConfig2(0x04, data);
			return status;
		}

		status GetEnable(Enable *enable)
		{
			ushort data;
			status status = mRegister->ReadConfig2(0x04, &data);
			if (status) return status;

			_ASSERTE(enable);
			enable->Memory = BIT_SHIFT_MASK(data, 1, 1) ? true : false;
			enable->Master = BIT_SHIFT_MASK(data, 2, 1) ? true : false;
		}

		status SetLatencyTimer(byte timer)
		{
			status status = mRegister->WriteConfig1(0x0d, timer);
			return status;
		}

		status GetLatencyTimer(byte *timer)
		{
			_ASSERTE(timer);
			status status = mRegister->ReadConfig1(0x0d, timer);
			return status;
		}

		status SetAddress(uint address)
		{
			status status = mRegister->WriteConfig4(0x10, address);
			return status;
		}

		status GetAddress(uint *address)
		{
			_ASSERTE(address);
			status status = mRegister->ReadConfig4(0x10, address);
			return status;
		}

		void Unlock()
		{
			#if defined(_DEBUG)
			UnlockUser();
			UnlockFactory();
			#endif
		}

	protected:
		XC3S::Register *const mRegister;
		uint mPTn;

		// \[XR[hJR: tbVیp̃R[hłʏ퓮ɂ͕sKvł邽
	};
}
}

#endif
